/* ================================================================
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL DINGOO GAMES OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ====================================================================
 */ 
#include "../inc/GameEngine.h"

#include "../game/game.h"
#include "../game/draw.h"
#include "../game/sysvid.h"

#include "../game/xcontrol.h"

#define		GF_MAINUI		1
#define		GF_GAMERUNNING	2

U32 tm, tmx;
unsigned short temp[SYSVID_WIDTH*SYSVID_HEIGHT];
char *gamePath;

//--------------------------------------------------------------------------------------------------------
GameEngine::GameEngine( void* userdata [] ) : 
					Engine( userdata )
{
  gamePath = (char *) userdata[2];

	m_pAudioData = NULL;
}
//--------------------------------------------------------------------------------------------------------
GameEngine::~GameEngine( void )
{
	Free();
}
//--------------------------------------------------------------------------------------------------------
/// init all resources
result	GameEngine::Init( void )
{
	InitDLRes();
	// 16 FPS
	SetFPS( 60 );

	//Init texture
	m_pImgS2d = new Texture();

	//Paint Area
	rcTest.left = 0;
	rcTest.top = 0;
	rcTest.right = 320;
	rcTest.bottom = 240;

	// Init game
  loadhighscores();

	
	m_pImgS2d  =new Texture();

	// Font Init
	m_StrMgr.Init( SimplifiedChinese, _LS("") );
	m_StrMgr.SetVRAMPtr( m_pDraw2D->GetVRAMPtr(), m_iScreenWidth, m_iScreenHeight );
	m_StrMgr.Load( _LS("string.sbn") );

	// Sound Init
  m_bUseAudio = true;
	m_pAudioData = new AudioData[24];
	m_SoundVol = 51;

	m_ScreenRatio = 1; // 1 = FullScreen / 0 = Original show

	m_Flag = GF_GAMERUNNING;
  m_MenuSel = m_MenuShowSound = 0;

  // Init game
	loaddata(); /* load cached data */
	game_period = sysarg_args_period ? sysarg_args_period : GAME_PERIOD;
	tm = sys_gettime();
	game_state = XRICK;

	return S2D_OK;
}
//--------------------------------------------------------------------------------------------------------
/// free all resources
void	GameEngine::Free( void )
{
	sys_shutdown();

	ReleaseDLRes();
  SAFE_DELETE_ARRAY( m_pAudioData );
}
//--------------------------------------------------------------------------------------------------------
/// game logic before render
void	GameEngine::Exec( void )
{  
  // timer 
  tmx = tm; tm = sys_gettime(); tmx = tm - tmx;
  if (tmx < game_period) sys_sleep(game_period - tmx);
}
//--------------------------------------------------------------------------------------------------------
/// game logic after render
void	GameEngine::Exec2( void )
{
  draw_STATUSRECT.next = NULL;  /* FIXME freerects should handle this */

  /* sound */
  /*snd_mix();*/

  // events 
  if (game_waitevt) {
    JoyPadControl();  /* wait for an event */
  }
  else {
    JoyPadControl();  /* process events (non-blocking) */
  }
    
  // frame 
  frame();
}
//--------------------------------------------------------------------------------------------------------
/// pre-render proccess
void	GameEngine::PreRender( void )
{

}
//--------------------------------------------------------------------------------------------------------
/// render
void	GameEngine::Render( void )
{
	if (m_Flag == GF_GAMERUNNING) {
	 DrawXRickGame();
	}
	else {
    DrawXRickMenu();
	}
}
//--------------------------------------------------------------------------------------------------------
void	GameEngine::PostRender( void )
{
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::DispDebugInfo( void )
{
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::EnterGame( s32 MapId )
{ 
}
//--------------------------------------------------------------------------------------------------------
void	GameEngine::PlayMusic(u32 ID,bool bRep)
{
	/*
  if(!m_bUseAudio){
		return;
	}
	if(ID>=m_AudioCnt){
		return;
	}*/
	m_pAudioPlayer->Play(&m_pAudioData[ID],0,m_pAudioData[ID].GetLength(),m_SoundVol,bRep);
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::StopMusic()
{
	m_pAudioPlayer->Stop(0);
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::PlaySound(u32 ID)
{
/*
	if(!m_bUseAudio){
		return;
	}
	if(ID>=m_AudioCnt){
		return;
	}
*/
	for(s32 i=1;i<AudioPlayer::MaxChannels;i++){
		if(m_pAudioPlayer->GetState(i) != AUDIO_PLAYING){
			m_pAudioPlayer->Play(&m_pAudioData[ID],i,m_pAudioData[ID].GetLength(),m_SoundVol,false);
			return;
		}
	}
}	

//--------------------------------------------------------------------------------------------------------
void	GameEngine::StopAllSound()
{
	m_bUseAudio = false;

	m_pAudioPlayer->StopAll();
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::ExitGame( void )
{
  StopAllSound();

	Exit();
}

//--------------------------------------------------------------------------------------------------------
void GameEngine::LoadCfg(void)
{
	// get full filename with path
	Char fullname[256];
	s2d::strcpy( fullname, _LS(".\\lltconfig.sdt") );

	FILE* m_fp = fopen( fullname, _LS("rb") );

	if(m_fp == NULL){
		return;
	}

	// read file to memory
	fseek( m_fp, 0, SEEK_END );
	s32 size =  ftell(m_fp);

	s32* mem = (s32*)  malloc( size );
	s32* pstar = mem;

	fseek(m_fp, 0, SEEK_SET );
	fread( mem, size,m_fp );
	fclose(m_fp);

	s32 Scene = *(s32*)mem;

	free( pstar );
}
//--------------------------------------------------------------------------------------------------------
void GameEngine::SaveCfg(void)
{
	// get full filename with path
	Char fullname[256];
	s2d::strcpy( fullname, _LS(".\\lltconfig.sdt") );

	FILE* m_fp = fopen( fullname, _LS("wb+") );

	if(m_fp == NULL){
		return;
	}

	s32 Scene=5;
	fwrite(&Scene, sizeof(s32),m_fp );

	// close file
	fclose(m_fp);
}

int pause=0; 

//--------------------------------------------------------------------------------------------------------
void GameEngine::JoyPadControl(void)
{
#define SETBIT(x,b) x |= (b)
#define CLRBIT(x,b) x &= ~(b)

	if (m_Flag == GF_MAINUI) {
	  if ( m_pInput->IsKeyClick( KEY_UP )) {
      m_MenuSel = (m_MenuSel == 0 ? 5 : m_MenuSel-1);
		  m_MenuShowSound = 0;
	  }
	  if ( m_pInput->IsKeyClick( KEY_DN )) {
		  m_MenuSel = (m_MenuSel == 5 ? 0 : m_MenuSel+1);
		  m_MenuShowSound = 0;
	  }
	  if ( m_pInput->IsKeyClick( KEY_LF )) {
		  if (m_MenuSel == 1) {
				switch (m_SoundVol) {
				  case   0 : m_SoundVol = 255; m_MenuShowSound = 11; break; // 100 %
			    case  25 : m_SoundVol =   0; m_MenuShowSound = 1; break; //   0 %
				  case  51 : m_SoundVol =  25; m_MenuShowSound = 2; break; //  10 %
				  case  76 : m_SoundVol =  51; m_MenuShowSound = 3; break; //  20 %
				  case 102 : m_SoundVol =  76; m_MenuShowSound = 4; break; //  30 %
				  case 127 : m_SoundVol = 102; m_MenuShowSound = 5; break; //  40 %
			    case 153 : m_SoundVol = 127; m_MenuShowSound = 6; break; //  50 %
				  case 178 : m_SoundVol = 153; m_MenuShowSound = 7; break; //  60 %
				  case 204 : m_SoundVol = 178; m_MenuShowSound = 8; break; //  70 %
				  case 229 : m_SoundVol = 204; m_MenuShowSound = 9; break; //  80 %
			    case 255 : m_SoundVol = 229; m_MenuShowSound = 10; break; //  90 %
				}
		  }
	  }
	  if ( m_pInput->IsKeyClick( KEY_RT )) {
		  if (m_MenuSel == 1) {
				switch (m_SoundVol) {
			    case   0 : m_SoundVol =  25; m_MenuShowSound = 2; break; //  10 %
			    case  25 : m_SoundVol =  51; m_MenuShowSound = 3; break; //  20 %
				  case  51 : m_SoundVol =  76; m_MenuShowSound = 4; break; //  30 %
				  case  76 : m_SoundVol = 102; m_MenuShowSound = 5; break; //  40 %
				  case 102 : m_SoundVol = 127; m_MenuShowSound = 6; break; //  50 %
			    case 127 : m_SoundVol = 153; m_MenuShowSound = 7; break; //  60 %
			    case 153 : m_SoundVol = 178; m_MenuShowSound = 8; break; //  70 %
				  case 178 : m_SoundVol = 204; m_MenuShowSound = 9; break; //  80 %
				  case 204 : m_SoundVol = 229; m_MenuShowSound = 10; break; //  90 %
				  case 229 : m_SoundVol = 255; m_MenuShowSound = 11; break; // 100 %
			    case 255 : m_SoundVol =   0; m_MenuShowSound = 1; break; //   0 %
				}
		  }
	  }
	  if ( m_pInput->IsKeyClick( KEY_1 )) {
		  m_MenuShowSound = 0;
		  switch (m_MenuSel) {
			case 0 : // Continue
				m_Flag = GF_GAMERUNNING;
				break;
			case 1 : // Volume
				break;
			case 2 : // Full Screen
				m_ScreenRatio = 1;
				rcTest.bottom = 240;
				m_Flag = GF_GAMERUNNING;
				break;
			case 3 : // Original show
				m_ScreenRatio = 0;
				rcTest.bottom = 200;
				m_Flag = GF_GAMERUNNING;
				break;
			case 4 : // Reset
				game_state = XRICK;
				m_Flag = GF_GAMERUNNING;
				break;
			case 5 : // Exit
      	        freedata(); /* free cached data */

				s2d::Exit(0);
				break;
		  }
	  }
	}
	else {
		if ( m_pInput->IsKeyDown( KEY_UP )) { SETBIT(control_status, CONTROL_UP); control_last = CONTROL_UP;  }
    else { CLRBIT(control_status, CONTROL_UP); control_last = CONTROL_UP; }
		if ( m_pInput->IsKeyDown( KEY_DN )) { SETBIT(control_status, CONTROL_DOWN); control_last = CONTROL_DOWN; }
    else { CLRBIT(control_status, CONTROL_DOWN); control_last = CONTROL_DOWN; }
		if ( m_pInput->IsKeyDown( KEY_RT )) { SETBIT(control_status, CONTROL_RIGHT); control_last = CONTROL_RIGHT; }
    else { CLRBIT(control_status, CONTROL_RIGHT); control_last = CONTROL_RIGHT; }
		if ( m_pInput->IsKeyDown( KEY_LF )) { SETBIT(control_status, CONTROL_LEFT); control_last = CONTROL_LEFT; }
    else { CLRBIT(control_status, CONTROL_LEFT); control_last = CONTROL_LEFT; }
		if ( m_pInput->IsKeyDown( KEY_1 )) { SETBIT(control_status, CONTROL_FIRE); control_last = CONTROL_FIRE; }
    else { CLRBIT(control_status, CONTROL_FIRE); control_last = CONTROL_FIRE; }
    
    /* For level select */
  	if ( m_pInput->IsKeyDown( KEY_2 )) { SETBIT(control_status, CONTROL_B); control_last = CONTROL_B; }
    else { CLRBIT(control_status, CONTROL_B); control_last = CONTROL_B; }  
  	if ( m_pInput->IsKeyDown( KEY_3 )) { SETBIT(control_status, CONTROL_X); control_last = CONTROL_X; }
    else { CLRBIT(control_status, CONTROL_X); control_last = CONTROL_X; }
  	if ( m_pInput->IsKeyDown( KEY_4 )) { SETBIT(control_status, CONTROL_Y); control_last = CONTROL_Y; }
    else { CLRBIT(control_status, CONTROL_Y); control_last = CONTROL_Y; } 
		if ( m_pInput->IsKeyDown( KEY_R1 ) && m_pInput->IsKeyDown( KEY_L1 ) ) {
	 		m_Flag = GF_MAINUI;
		}
    else if ( m_pInput->IsKeyDown( KEY_R1 ) ) {
      if (!pause){
        SETBIT(control_status, CONTROL_PAUSE);
        control_last = CONTROL_PAUSE;
        pause=1;
      }
      else {
        CLRBIT(control_status, CONTROL_PAUSE);
        control_last = CONTROL_PAUSE;
        pause=0;
      }
    } 
	}
}
//--------------------------------------------------------------------------------------------------------
void GameEngine::DrawXRickMenu(void)
{
#define FONTINC 20
#define FONTDEB 20
    int i;
	Point p(m_iScreenWidth>>1,0);
	Pixel colorwhite;colorwhite.rgb(0xff,0xff,0xff );
	Pixel colorred;colorred.rgb(0xff,0x00,0x00);
	Rect  rcSmall;
	rcSmall.left = 160;
	rcSmall.top = FONTDEB+20;
	rcSmall.right = 160+150;
	rcSmall.bottom = FONTDEB+140;

	m_pDraw2D->Clear(0,0,0);
	p.x = 160; p.y = 4;	m_StrMgr.DrawString( 1, colorwhite,ANCHOR_HCENTER|ANCHOR_TOP,p);

	for (i=1;i<7;i++) {
		p.x = 10; p.y = FONTDEB+FONTINC*i;	m_StrMgr.DrawString( i+1, (m_MenuSel == i-1 ? colorred : colorwhite),ANCHOR_LEFT|ANCHOR_TOP,p);
	}
	if (m_MenuShowSound) {
      p.x = 70; p.y = FONTDEB+FONTINC*2;	m_StrMgr.DrawString( m_MenuShowSound+7, colorred ,ANCHOR_LEFT|ANCHOR_TOP,p);
	}
  m_pImgS2d->CreateFromMemory(TF_TRUECOLOR,SYSVID_WIDTH,SYSVID_HEIGHT,&temp,sizeof(unsigned short),NULL);
  m_pDraw2D->SetTexture( m_pImgS2d );		
  m_pDraw2D->DrawStretchOverlay(rcSmall,NULL);
  m_pImgS2d->Free();
}

//--------------------------------------------------------------------------------------------------------
#define RGB_565(_r,_g,_b) (((_r>>3)<<11) | ((_g>>2)<<5) | (_b>>3))
void GameEngine::DrawXRickGame(void)
{
  unsigned int inX,inY,inColor;

  for (inY=0;inY<SYSVID_HEIGHT;inY++) {
    for (inX=0;inX<SYSVID_WIDTH;inX++) {
      inColor = sysvid_fb[(inY * SYSVID_WIDTH) + inX];
      temp[(inY * SYSVID_WIDTH) + inX] = RGB_565(palette[inColor].r,palette[inColor].g,palette[inColor].b);
    }
  }

  m_pImgS2d->CreateFromMemory(TF_TRUECOLOR,SYSVID_WIDTH,SYSVID_HEIGHT,&temp,sizeof(unsigned short),NULL);
  m_pDraw2D->SetTexture( m_pImgS2d );		
  m_pDraw2D->DrawStretchOverlay(rcTest,NULL);
  m_pImgS2d->Free();
}

//--------------------------------------------------------------------------------------------------------
void GameEngine::LoadMusic(int noMus, char *name)
{
  char sz[64];
  
  sprintf(sz,"audio\\%s",name);
  m_pAudioData[noMus].LoadPCM( _LS(sz) );
}
